// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the map-system's wraparound feature.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
volatile u8 ATTR_EWRAM g_NewFrame=TRUE;
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void ScrollMap();
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // Initialize the Map-System. If you do not initialize
    // the Map-System before calling any Map related function,
    // the program behaviour is undefined.
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    // Load Tileset into VideoRAM
    ham_bg[0].ti = ham_InitTileSet(ResData(RES_WRAPMETAL_RAW), RES_WRAPMETAL_RAW_SIZE16, 1, TRUE);
    
    // Create a map for background 0
    hel_MapCreate(0,                            // Background number
                  128/8,                        // width in tiles
                  128/8,                        // height in tiles
                  ResData(RES_WRAPMETAL_MAP),   // Pointer to source MapData
                  sizeof(u16),                  // DataTypeSize of one element from Source MapData
                  MAP_FLAGS_DEFAULT);           // Flags to control map behaviour
 
    // Enable Wrapping. Wraparound happens when the
    // map is not limited to the bounds anymore, so
    // we just remove bounds checking by specifying MAP_BOUNDS_NONE!
    hel_MapSetBounds(0, 0, 0, FIXED_FROMINT(240), FIXED_FROMINT(160), MAP_BOUNDS_NONE);
    
    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_BG_PAL));
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // Infinite loop
    for(;;)
    {
        hel_PadCapture();
        ScrollMap();
        
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}


// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(3.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

   // Now Scroll the map!
   hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
    hel_IntrAcknowledge(INT_TYPE_VBL);
}

