// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 17th June 2007
//
// Purpose:       Demo shows how to use the Object Rotation/Scaling System.
//                This demo is a HEL Library test program actually, to
//                make sure hel_ObjGetRotScale works as it should be.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"


// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct TRotationObject
{
  THandle ObjNo;  // Handle of Object
  THandle RotNo;  // Handle of Rotation/Scaling Set
  u16 Angle;      // Helper variable for angle
  s16 Zoom;       // Helper variable for zoom
}TRotationObject;

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// Reserve memory for the Object-Rotation/Scaling-System. This buffer
// is used to manage internal states of the Object-Rotation/Scaling-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjRotScaleSystemBuffer[HEL_SUBSYSTEM_OBJROTSCALE_REQUIREDMEMORY];

// The rotation object
TRotationObject ATTR_EWRAM g_Obj;


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void InitObject(void);
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Init the Object Rotation/Scaling System
    hel_ObjRotScaleInit(g_ObjRotScaleSystemBuffer);
    
    // init objects
    InitObject();
    
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
		// Update Angle
		g_Obj.Angle += 1;

		// Update Rotation and Scaling
		hel_ObjRotScaleUpdate(g_Obj.RotNo, g_Obj.Angle % 360, g_Obj.Zoom, g_Obj.Zoom);

		// Wait for VBlank
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Create object and rotation/scaling sets
// -----------------------------------------------------------------------------
void InitObject(void)
{
	THandle hRot;

	// Create Object
	g_Obj.ObjNo = hel_ObjCreate256
		(
			ResData(RES_HEART_RAW), // Pointer to graphic data
			OBJ_SHAPE_SQUARE,       // Object Shape
			1,                      // Object Size
			OBJ_MODE_NORMAL,        // Object Mode
			120-8,					// X
			80-8                    // Y
		);

	g_Obj.Angle = 0;		// no rotation
	g_Obj.Zoom  = 0x100;	// (0x100 = no zoom)

	// Create Rotation/Scaling Set
	g_Obj.RotNo = hel_ObjRotScaleCreate(g_Obj.Angle, g_Obj.Zoom, g_Obj.Zoom);

	// Enable Rotation/Scaling for the Object
	hel_ObjSetRotScale(g_Obj.ObjNo, TRUE);

	// Enable Double size
	hel_ObjSetDblSize(g_Obj.ObjNo, TRUE);

	// Select the Rotation/Scaling Set for the Object
	hel_ObjSelectRotScale(g_Obj.ObjNo, g_Obj.RotNo);



	// Ignore the following code, it just tests if
	// hel_ObjGetRotScale works correctly.

	hRot = hel_ObjGetRotScale(g_Obj.ObjNo);
	HEL_DEBUG_MSG("Rot/Scale Handle %d", hRot);
	HEL_ASSERT(hRot != HANDLE_INVALID, "Error, handle must exist.");

	// turn off rot/scale
	hel_ObjSetRotScale(g_Obj.ObjNo, FALSE);
	hRot = hel_ObjGetRotScale(g_Obj.ObjNo);
	HEL_DEBUG_MSG("Rot/Scale Handle %d", hRot);
	HEL_ASSERT(hRot == HANDLE_INVALID, "Error, handle must not exist.");

	// turn on rot/scale
	hel_ObjSetRotScale(g_Obj.ObjNo, TRUE);
	hRot = hel_ObjGetRotScale(g_Obj.ObjNo);
	HEL_DEBUG_MSG("Rot/Scale Handle %d", hRot);
	HEL_ASSERT(hRot == HANDLE_INVALID, "Error, handle must not exist.");

	// select a rot/scale again
	hel_ObjSelectRotScale(g_Obj.ObjNo, g_Obj.RotNo);
	hRot = hel_ObjGetRotScale(g_Obj.ObjNo);
	HEL_DEBUG_MSG("Rot/Scale Handle %d", hRot);
	HEL_ASSERT(hRot != HANDLE_INVALID, "Error, handle must exist.");
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();
  
   // Transmit Object Rotation/Scaling Attributes to hardware
   hel_ObjRotScaleTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}

