// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the repeat delay/rate
//						    Pad System feature.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------

// The background number the
// background Text System uses.
#define BGNO        		0

// amount of available menu items
// (has no impact on creation, it wil
// not create more text entries if you
// change this define. It has only impact
// on the object)
#define MAX_MENUITEMS   8

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

THandle ATTR_EWRAM g_Handle;
s8 ATTR_EWRAM g_CurMenuItem=0;

// A buffer of memory which the Background Text
// System uses to manage internal states.
// It must not be modified from outside as long as
// any Background Text System is running.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_BgTextSystemMemory[HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY];

// Reserve memory for the Obj-System. This buffer
// is used to manage internal states of the Obj-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// g_CharLUT will hold offsets to each character
// in the map data. This array basically functions
// as a lookup table. It must be 256 elements big,
// which equals the maximum number of a characters
// in ASCII (0..255).
// This buffer is only neccessary when you pass
// BGTEXT_FLAGS_GENERATELUT to hel_BgTextCreate.
// Please see documentation for details.
u16 ATTR_EWRAM g_CharLUT[256];

// That's the order of characters how they
// appear on the source graphic.
// See hel/demos/sharedmedia/charset8x8.bmp
const char CHARORDER[] =
    " BCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmn"\
    "opqrstuvwxyz0123456789?,.-;:A#^+*@"\
    "!\"%\%&/()=?|\\<>[]{}";


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void SetupPadSystem(void);
void SetupTextSystem(void);
void SetupObjSystem(void);
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
	ham_Init();
	hel_SysSetPrefetch(TRUE);
	ham_SetBgMode(0);

   SetupPadSystem();
	SetupObjSystem();
	SetupTextSystem();

	hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);

	// infinite loop to hold
	// the application alive
	for(;;)
	{
			hel_PadCapture();

			if(hel_PadQuery()->Pressed.Down)
			{
            g_CurMenuItem++;
				if(g_CurMenuItem >= MAX_MENUITEMS)
					g_CurMenuItem = 0;
			}

			if(hel_PadQuery()->Pressed.Up)
			{
            g_CurMenuItem--;
				if(g_CurMenuItem < 0)
					g_CurMenuItem = MAX_MENUITEMS-1;
			}

			hel_ObjSetXY(g_Handle, 3*8 - 3, 5*8 + g_CurMenuItem * 8);

      if(hel_PadQuery()->Held.Up)
        hel_BgTextPrintF(BGNO, 1, 14, 0, "Up ticks..: %4d\0", hel_PadGetTicks(PAD_BUTTON_UP));

      if(hel_PadQuery()->Held.Down)
				hel_BgTextPrintF(BGNO, 1, 15, 0, "Down ticks: %4d\0", hel_PadGetTicks(PAD_BUTTON_DOWN));


      // Wait for Vertical Blank
      hel_SwiVBlankIntrWait();
	}

	return 0;
}

// -----------------------------------------------------------------------------
// Setup Pad System
// -----------------------------------------------------------------------------
void SetupPadSystem(void)
{
	hel_PadInit();
	hel_PadSetRepeatDelay(PAD_BUTTON_ALL, 32);
	hel_PadSetRepeatRate(PAD_BUTTON_ALL, 8);
}

// -----------------------------------------------------------------------------
// Setup Object System
// -----------------------------------------------------------------------------
void SetupObjSystem(void)
{
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);

    // Create an object!
    // You can also use hel_ObjCreate256 to create a 256 color object,
    // or hel_ObjCreate16 to create objects with 16 colors.
    g_Handle = hel_ObjCreate(
                     ResData(RES_SMALLCIRCLE_RAW), // Pointer to source graphic
                     OBJ_SHAPE_SQUARE,       // Obj Shape
                     0,                      // Obj Size, 1 means 16x16 pixels, if Shape is set to SQUARE
                     OBJ_MODE_NORMAL,        // Obj Mode
                     COLORS_256,             // Use 256 color mode
                     0,                      // Palette number. Only neccessary in 16 color mode
                     FALSE,                  // Don't use mosaic
                     FALSE,                  // Don't flip the sprite horizontally
                     FALSE,                  // Don't flip the object vertically
                     0,                      // Priority against background. 0=higest
                     FALSE,                  // Don't make the object double-sized
                     241,                    // Destination horzintal screen coordinate in pixels
                     161                     // Destination vertical screen coordinate in pixels
                  );

    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
}

// -----------------------------------------------------------------------------
// Setup Text System
// -----------------------------------------------------------------------------
void SetupTextSystem(void)
{
    // Initialize the Background Text System.
    // The g_BgTextSystemMemory array is used by the
    // Background Text System to manage internal states
    // and must not be modified from outside as long as
    // any Background Text System is running.
    hel_BgTextInit((void*)g_BgTextSystemMemory);

    // Initialize the tileset and an empty
    // mapset using regular HAMlib functions
    ham_bg[BGNO].ti = ham_InitTileSet(ResData(RES_CHARSET8X8_RAW), RES_CHARSET8X8_RAW_SIZE16, COLORS_16, TRUE);
    ham_bg[BGNO].mi = ham_InitMapEmptySet(MAP_SIZE_32X32, FALSE);

    // Create a Background Text System
    hel_BgTextCreate(
        BGNO,   // BgNo
        1,      // Width of one character specified in tiles
        1,      // Height of one character specified in tiles
        ResData(RES_CHARSET8X8_MAP),    // Pointer to character map
        CHARORDER,      // Pointer to array that reflects the order of characters on source graphic
        g_CharLUT,      // Pointer to buffer that receives the generated lookup table
        BGTEXT_FLAGS_GENERATELUT);  // Flags that control creation and print behaviour

    // Print text on background.
    // The "\n" sequence forces a new line,
    // so you can write several lines with
    // only one call to hel_BgTextPrint
    hel_BgTextPrint(BGNO,
        4,      // X
        5,      // Y
        0,      // Flags, must be zero at this time
        "Option 1\n"\
        "Option 2\n"\
        "Option 3\n"\
        "Option 4\n"\
		    "Option 5\n"\
		    "Option 6\n"\
    	  "Option 7\n"\
        "Option 8\0"     // NULL-terminated string to print
        );

    hel_BgTextPrint(BGNO, 1, 1, 0, "Press/Hold Up/Down button!\0");

    // Load palette data to Vram
    hel_PalBgLoad16(ResData16(RES_BG15_PAL), 15);

    // Initialize and show background
    ham_InitBg(BGNO, TRUE, 0, FALSE);
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}


